#!/usr/bin/python -tt
# Matthieu Pichene

"""
Created on Thu Jan 22 17:40:18 2015

@author: matthieupichene
"""
import re
import os
import sys
import time
import FitMatrix
import numpy as np
import copy
#from multiprocessing import Pool
from multiprocessing import Process, Manager
#a = [[1,2,3],[4,5,6],[7,8,9,10]]
#list(itertools.product(*a))
from ast import literal_eval

class TreeNode(object): # Tree node
    def __init__(self):
        self.parent = None
        self.children = []
        self.data = None
        self.rank = 0
        self.treenumber = None
        
def SetChild(PNode,CNode):
    """   
    Set the child of a node object. The child receive the node as a parent.
    """
    if CNode not in PNode.children:
        PNode.children.append(CNode)
    if CNode.parent:
        CNode.parent.children.remove(CNode) #remove child in the list of children of the last parent
    CNode.parent = PNode
    #CNode.rank = PNode.rank+1
  
def RepairTree(PNode,CNode):
    """   
    Recursively transform a part of the tree to avoid multiple parents
    """
    if PNode.parent:
        RepairTree(PNode.parent,PNode)
    SetChild(CNode,PNode)
    
    
def GenerateClusterTree(clusters):
    Treedict = {}
    for cluster in clusters:
        if len(cluster)==1:
            Tnode = TreeNode()
            Tnode.data = cluster[0]
            Treedict[cluster[0]] = Tnode
        elif len(cluster)==2:
            if Treedict.has_key(cluster[0]):
                parent = Treedict[cluster[0]]
            else:
                parent = TreeNode()
                parent.data = cluster[0]
                Treedict[cluster[0]] = parent
            if Treedict.has_key(cluster[1]):
                child = Treedict[cluster[1]]
            else:
                child = TreeNode()
                child.data = cluster[1]
                Treedict[cluster[1]] = child
            if child.parent:
                RepairTree(parent,child)
            else:
                SetChild(parent,child)
        else:
            sys.exit('ERROR : Cluster of more than 2 species')
    return Treedict
    
def FindSeparatedTrees(Treedict):
    """   
    Give all nodes a tree number and a rank in the tree
    """
    Treenumber = 0
    for TNode in Treedict.keys():
        if not Treedict[TNode].parent:
            GiveNodeRank(Treedict[TNode],0,Treenumber)
            Treenumber += 1

def GiveNodeRank(TNode,rank,number):
    """   
    Find recursively the rank of all nodes in a tree, give the same tree
    number to all those nodes
    """
    TNode.rank = rank
    TNode.treenumber = number
    for child in TNode.children:
        GiveNodeRank(child,rank+1,number)

def readtable(Tablesfile):
    """   
    Import table as a list of arrays
    """

    Tablesread = open(Tablesfile, 'rU')
    Tablestext = Tablesread.read()
    Tablesread.close()
    #Tablesval = re.findall(r'(\d+)( \d+\.?\d*)+\n',Tablestext)
    Tablesval = []
    Tableslines=re.findall(r'([^\n]*)\n',Tablestext) #Should be a more efficient way
    for Line in Tableslines:
        Tablesval.append(re.split(' ',Line))
    Tablesarray =[]
    for table in Tablesval:
        Tableval = []
        for value in table:
            Tableval.append(value)
        Tablesarray.append(Tableval)
    return Tablesarray
    
def initialvalues(Resultdir,init):
    """   
    Create the initial table.
    """
    if not os.path.exists(Resultdir):
        os.mkdir(Resultdir) 
    initread= open(init,'r')
    initlines = initread.readlines()
    initread.close()
        
    Tableswrite = open(Resultdir+'T0.txt', 'w')
    for i in initlines:
        ilist = i.split()
        states = len(ilist)-1
        for j in range(states):
            Tableswrite.write(ilist[0]+' '+str(j)+' '+ilist[j+1]+'\n') 
    Tableswrite.close()

    
def FindTreePath(CPT,Links,Cluster,ClusterDict,Species,Values,states):
    """   
    Return the best path of the tree to obtain a list for all values of all 
    species of interest 
    """
    rank = []
    tree = []
    value = []
    result = []
    for link in Links :
        if link[0] == Cluster[0]:
            link1 = link[1:]
        if len(Cluster) == 2:
            if link[0] == Cluster[1]:
                link2 = link[1:]
        else:
            link2 = []
    for specie in Species:
        rank.append(ClusterDict[specie].rank)
        tree.append(ClusterDict[specie].treenumber)
        value.append(ClusterDict[specie].data)
    for i in list(set(tree)): #do each tree separately 
        trank = []
        tvalue = []
        for j in range(len(tree)):
            if tree[j]==i:
                trank.append(rank[j])
                tvalue.append(value[j])   
        actualrank = max(trank)
        foundall = False
        parentdict = {}
        while not foundall:
            toadd = []
            for k in range(len(trank)):
                if trank[k] == actualrank:
                    if len(trank)==1 : # case only one value to do.                       
                        foundall =  True
                        if parentdict == {}: #tree of one
                            KeepValues = FindKeepValue(tvalue[k],link1,link2,Cluster,CPT)
                            KeepProba = []
                            for KValue in Values[tvalue[k]]:
                                if [KValue[0]] in KeepValues:
                                    KeepProba.append(KValue) 
                                    
                            #parentdict[tvalue[k]]=[tvalue[k],Values[tvalue[k]]]
                            parentdict[tvalue[k]]=[tvalue[k],KeepProba]
                            result.append([str([tvalue[k]]),KeepProba])
    
                        elif tvalue[k] not in value: #found the root, root not needed
                            Newset = Marginalize(parentdict[tvalue[k]],tvalue[k],states)
                            parentdict[Newset[0]]= Newset[1]
                            result.append(Newset)
                        else: #fount the root, root needed
                            result.append(parentdict[tvalue[k]])
                            
                    else: #root not found
                        child = tvalue[k]
                        parent = ClusterDict[child].parent.data
                        if parent not in tvalue and parent not in toadd: 
                            toadd.append(parent) 
                        clustercp = [child,parent]
                        clustercp.sort()
                        
                        
                        if clustercp[0] not in value:
                            if clustercp[1] not in value: #check if parent in kept values
                                KeepValues = [[x] for x in range(states)]
                                KeepPos = 0
                            else:
                                KeepValues = FindKeepValue(clustercp[1],link1,link2,Cluster,CPT)
                                KeepPos = 1
                        else:
                            if clustercp[1] not in value: #check if parent in kept values
                                KeepValues = FindKeepValue(clustercp[0],link1,link2,Cluster,CPT)
                                KeepPos = 0
                            else:
                                KeepValues = FindKeepCluster(clustercp[0],clustercp[1],link1,link2,Cluster,CPT)
                                KeepPos = 2
                                
                        clustercp = str(clustercp)         
                        #Clustervalue = Values[clustercp] #Old
                        KeepProba = [] #new clustervalue
    
                        for KValue in Values[clustercp]:
                            valuelist = powertolist(KValue[0],states,2)
                            if KeepPos == 0:
                                if [valuelist[0]] in KeepValues:
                                    KeepProba.append(KValue)
                            elif KeepPos == 1:
                                if [valuelist[1]] in KeepValues:
                                    KeepProba.append(KValue)
                            else:
                                if valuelist in KeepValues:
                                    KeepProba.append(KValue)
                                    
                        if tvalue[k] not in parentdict.keys() : # no childrens         
                            if not parentdict.has_key(parent):
                                parentdict[parent]=[clustercp,KeepProba]
                            else: #fusion needed
                                cluster1= parentdict[parent]
                                cluster2= [clustercp,KeepProba]
                                Fusionval=[parent,Values[parent]] #TODO
                                parentdict[parent] = FusionCluster(cluster1,cluster2,Fusionval,states)
    
                        else: # has children
                            childcluster = parentdict[child] #cluster of children
                            parentcluster= [clustercp,KeepProba]
                            Fusionval=[tvalue[k],Values[tvalue[k]]]
                            Fusedcluster = FusionCluster(childcluster,parentcluster,Fusionval,states)
                            if tvalue[k] not in value:
                                Fusedcluster = Marginalize(Fusedcluster,tvalue[k],states)
                            if not parentdict.has_key(parent):
                                parentdict[parent] = Fusedcluster
                                
                            else :
                                cluster1= parentdict[parent]
                                cluster2= Fusedcluster
                                Fusionval=[parent,Values[parent]] #TODO
                                #Fusionval=[tvalue[k],Values[tvalue[k]]]
                                parentdict[parent] = FusionCluster(cluster1,cluster2,Fusionval,states)
                                

                            
            #generate the new temporary ranks and values
            ttrank = []
            ttvalue = []
            for l in range (len(trank)):
                if trank[l] != actualrank:
                    ttrank.append(trank[l])
                    ttvalue.append(tvalue[l])
            for m in toadd:
                ttrank.append(actualrank -1)
                ttvalue.append(m)
            trank = ttrank
            tvalue = ttvalue
            actualrank -=1

    return result

def FindTreePath2(CPT,Links,Cluster,ClusterDict,Species,Values,states):
    """   
    Return the best path of the tree to obtain a list for all values of all 
    species of interest 
    """
    rank = []
    tree = []
    value = []
    result = []
    for link in Links :
        if link[0] == Cluster[0]:
            link1 = link[1:]
        if len(Cluster) == 2:
            if link[0] == Cluster[1]:
                link2 = link[1:]
        else:
            link2 = []
    for specie in Species:
        rank.append(ClusterDict[specie].rank)
        tree.append(ClusterDict[specie].treenumber)
        value.append(ClusterDict[specie].data)
    for i in list(set(tree)): #do each tree separately 
        trank = []
        tvalue = []
        for j in range(len(tree)):
            if tree[j]==i:
                trank.append(rank[j])
                tvalue.append(value[j])   
        actualrank = max(trank)
        foundall = False
        parentdict = {}
        while not foundall:
            toadd = []
            for k in range(len(trank)):
                if trank[k] == actualrank:
                    if len(trank)==1 : # case only one value to do.                       
                        foundall =  True
                        if parentdict == {}: #tree of one
                            KeepValues = FindKeepValue(tvalue[k],link1,link2,Cluster,CPT)
                            KeepProba = []
                            for KValue in Values[tvalue[k]]:
                                if [KValue[0]] in KeepValues:
                                    KeepProba.append(KValue) 
                                    
                            #parentdict[tvalue[k]]=[tvalue[k],Values[tvalue[k]]]
                            parentdict[tvalue[k]]=[tvalue[k],KeepProba]
                            result.append([str([tvalue[k]]),KeepProba])

                        elif tvalue[k] not in value: #found the root, root not needed
                            Newset = Marginalize(parentdict[tvalue[k]],tvalue[k],states)
                            parentdict[Newset[0]]= Newset[1]
                            result.append(Newset)
                        else: #fount the root, root needed
                            result.append(parentdict[tvalue[k]])
                            
                    else: #root not found
                        child = tvalue[k]
                        parent = ClusterDict[child].parent.data
                        if parent not in tvalue and parent not in toadd: 
                            toadd.append(parent) 
                        clustercp = [child,parent]
                        clustercp.sort()
                        
                        
                        if clustercp[0] not in value:
                            if clustercp[1] not in value: #check if parent in kept values
                                KeepValues = [[x] for x in range(states)]
                                KeepPos = 0
                            else:
                                KeepValues = FindKeepValue(clustercp[1],link1,link2,Cluster,CPT)
                                KeepPos = 1
                        else:
                            if clustercp[1] not in value: #check if parent in kept values
                                KeepValues = FindKeepValue(clustercp[0],link1,link2,Cluster,CPT)
                                KeepPos = 0
                            else:
                                KeepValues = FindKeepCluster(clustercp[0],clustercp[1],link1,link2,Cluster,CPT)
                                KeepPos = 2
                                
                        clustercp = str(clustercp)         
                        #Clustervalue = Values[clustercp] #Old
                        KeepProba = [] #new clustervalue

                        for KValue in Values[clustercp]:
                            valuelist = powertolist(KValue[0],states,2)
                            if KeepPos == 0:
                                if [valuelist[0]] in KeepValues:
                                    KeepProba.append(KValue)
                            elif KeepPos == 1:
                                if [valuelist[1]] in KeepValues:
                                    KeepProba.append(KValue)
                            else:
                                if valuelist in KeepValues:
                                    KeepProba.append(KValue)
                                    
                        if tvalue[k] not in parentdict.keys() : # no childrens         
                            if not parentdict.has_key(parent):
                                parentdict[parent]=[clustercp,KeepProba]
                            else: #fusion needed
                                cluster1= parentdict[parent]
                                cluster2= [clustercp,KeepProba]
                                Fusionval=[parent,Values[parent]] #TODO
                                parentdict[parent] = FusionCluster(cluster1,cluster2,Fusionval,states)
    
                        else: # has children
                            childcluster = parentdict[child] #cluster of children
                            parentcluster= [clustercp,KeepProba]
                            Fusionval=[tvalue[k],Values[tvalue[k]]]
                            Fusedcluster = FusionCluster(childcluster,parentcluster,Fusionval,states)
                            if tvalue[k] not in value:
                                Fusedcluster = Marginalize(Fusedcluster,tvalue[k],states)
                            if not parentdict.has_key(parent):
                                parentdict[parent] = Fusedcluster
                                
                            else :
                                cluster1= parentdict[parent]
                                cluster2= Fusedcluster
                                Fusionval=[parent,Values[parent]] #TODO
                                #Fusionval=[tvalue[k],Values[tvalue[k]]]
                                parentdict[parent] = FusionCluster(cluster1,cluster2,Fusionval,states)
                                

                            
            #generate the new temporary ranks and values
            ttrank = []
            ttvalue = []
            for l in range (len(trank)):
                if trank[l] != actualrank:
                    ttrank.append(trank[l])
                    ttvalue.append(tvalue[l])
            for m in toadd:
                ttrank.append(actualrank -1)
                ttvalue.append(m)
            trank = ttrank
            tvalue = ttvalue
            actualrank -=1

    return result
    
def FindTreePathOld(CPT,CPTsAr,Links,Cluster,ClusterDict,Species,Values,states):
    """   
    Return the best path of the tree to obtain a list for all values of all 
    species of interest 
    """
    rank = []
    tree = []
    value = []
    result = []
    for link in Links :
        if link[0] == Cluster[0]:
            link1 = link[1:]
        if len(Cluster) == 2:
            if link[0] == Cluster[1]:
                link2 = link[1:]
        else:
            link2 = []
    for specie in Species:
        rank.append(ClusterDict[specie].rank)
        tree.append(ClusterDict[specie].treenumber)
        value.append(ClusterDict[specie].data)
    for i in list(set(tree)): #do each tree separately 
        trank = []
        tvalue = []
        for j in range(len(tree)):
            if tree[j]==i:
                trank.append(rank[j])
                tvalue.append(value[j])   
        actualrank = max(trank)
        foundall = False
        parentdict = {}
        while not foundall:
            toadd = []
            for k in range(len(trank)):
                if trank[k] == actualrank:
                    if len(trank)==1 : # case only one value to do.                       
                        foundall =  True
                        if parentdict == {}: #tree of one
                            KeepValues = FindKeepValue(tvalue[k],link1,link2,Cluster,CPT)
                            KeepProba = []
                            for KValue in Values[tvalue[k]]:
                                if [KValue[0]] in KeepValues:
                                    KeepProba.append(KValue) 
                                    
                            #parentdict[tvalue[k]]=[tvalue[k],Values[tvalue[k]]]
                            parentdict[tvalue[k]]=[tvalue[k],KeepProba]
                            result.append([str([tvalue[k]]),KeepProba])

                        elif tvalue[k] not in value: #found the root, root not needed
                            Newset = Marginalize(parentdict[tvalue[k]],tvalue[k],states)
                            parentdict[Newset[0]]= Newset[1]
                            result.append(Newset)
                        else: #fount the root, root needed
                            result.append(parentdict[tvalue[k]])
                            
                    else: #root not found
                        child = tvalue[k]
                        parent = ClusterDict[child].parent.data
                        if parent not in tvalue and parent not in toadd: 
                            toadd.append(parent) 
                        clustercp = [child,parent]
                        clustercp.sort()
                        
                        
                        if clustercp[0] not in value:
                            if clustercp[1] not in value: #check if parent in kept values
                                KeepValues = [[x] for x in range(states)]
                                KeepPos = 0
                            else:
                                KeepValues = FindKeepValue(clustercp[1],link1,link2,Cluster,CPT)
                                KeepPos = 1
                        else:
                            if clustercp[1] not in value: #check if parent in kept values
                                KeepValues = FindKeepValue(clustercp[0],link1,link2,Cluster,CPT)
                                KeepPos = 0
                            else:
                                KeepValues = FindKeepCluster(clustercp[0],clustercp[1],link1,link2,Cluster,CPT)
                                KeepPos = 2
                                
                        clustercp = str(clustercp)         
                        #Clustervalue = Values[clustercp] #Old
                        KeepProba = [] #new clustervalue

                        for KValue in Values[clustercp]:
                            valuelist = powertolist(KValue[0],states,2)
                            if KeepPos == 0:
                                if [valuelist[0]] in KeepValues:
                                    KeepProba.append(KValue)
                            elif KeepPos == 1:
                                if [valuelist[1]] in KeepValues:
                                    KeepProba.append(KValue)
                            else:
                                if valuelist in KeepValues:
                                    KeepProba.append(KValue)
                                    
                        if tvalue[k] not in parentdict.keys() : # no childrens         
                            if not parentdict.has_key(parent):
                                parentdict[parent]=[clustercp,KeepProba]
                            else: #fusion needed
                                cluster1= parentdict[parent]
                                cluster2= [clustercp,KeepProba]
                                Fusionval=[parent,Values[parent]] #TODO
                                parentdict[parent] = FusionCluster(cluster1,cluster2,Fusionval,states)
                                parentdict[parent] = ClearCPT(parentdict[parent],CPTsAr,Cluster,Links,states)
                        else: # has children
                            childcluster = parentdict[child] #cluster of children
                            parentcluster= [clustercp,KeepProba]
                            Fusionval=[tvalue[k],Values[tvalue[k]]]
                            Fusedcluster = FusionCluster(childcluster,parentcluster,Fusionval,states)
                            if tvalue[k] not in value:
                                Fusedcluster = Marginalize(Fusedcluster,tvalue[k],states)
                            if not parentdict.has_key(parent):
                                parentdict[parent] = Fusedcluster
                                parentdict[parent] = ClearCPT(parentdict[parent],CPTsAr,Cluster,Links,states)
                            else :
                                cluster1= parentdict[parent]
                                cluster2= Fusedcluster
                                Fusionval=[parent,Values[parent]] #TODO
                                #Fusionval=[tvalue[k],Values[tvalue[k]]]
                                parentdict[parent] = FusionCluster(cluster1,cluster2,Fusionval,states)
                                parentdict[parent] = ClearCPT(parentdict[parent],CPTsAr,Cluster,Links,states)

                            
            #generate the new temporary ranks and values
            ttrank = []
            ttvalue = []
            for l in range (len(trank)):
                if trank[l] != actualrank:
                    ttrank.append(trank[l])
                    ttvalue.append(tvalue[l])
            for m in toadd:
                ttrank.append(actualrank -1)
                ttvalue.append(m)
            trank = ttrank
            tvalue = ttvalue
            actualrank -=1

    return result
    
def FindReducedTree(Links,Cluster,ClusterDict,Species):
    """   
    Return the best path of the tree to obtain a list for all values of all 
    species of interest 
    """
    #Is CPTcount useful ?
    rank = []
    tree = []
    value = []
    CPTcount = []
    result = []
    
    for link in Links :
        if link[0] == Cluster[0]:
            link1 = link[1:]
        if len(Cluster) == 2:
            if link[0] == Cluster[1]:
                link2 = link[1:]
        else:
            link2 = []
    for specie in Species:
        rank.append(ClusterDict[specie].rank)
        tree.append(ClusterDict[specie].treenumber)
        value.append(ClusterDict[specie].data)
        CPTcount.append(1 * (specie in link1) + 2 * (specie in link2))
    for i in list(set(tree)): #do each tree separately 
        trank = []
        tvalue = []
        tCPTcount = []
        for j in range(len(tree)):
            if tree[j]==i:
                trank.append(rank[j])
                tvalue.append(value[j])  
                tCPTcount.append(CPTcount[j])
        actualrank = max(trank)
        foundall = False
        while not foundall:
            toadd = []
            for k in range(len(trank)): #TODO : Inverted. Can cause crash ?
                if trank[k] == actualrank: #Only do last rank 
                    if len(trank)==1 : # case only one value to do.                       
                        foundall =  True
                        result.append(tvalue[k])
 
                    else: #root not found
                        child = tvalue[k]
                        childval = tCPTcount[k]
                        parent = ClusterDict[child].parent.data
                        
                        
                        if childval == 3:
                            result.append(tvalue[k])
                            if parent not in tvalue and parent not in toadd: 
                                toadd.append(parent)
                        else :
                            if parent not in tvalue:
                                if parent in toadd:
                                    toadd.remove(parent)
                                trank[k] -= 1
                                tvalue[k] = parent
                                    #trank[k]
                                
                                    
                                #tCPTcount.append(tCPTcount[k])
                            else:
                                l = tvalue.index(parent)
                                if tCPTcount[k] != tCPTcount[l]:
                                    tCPTcount[l] = min(tCPTcount[k] + tCPTcount[l],3)
                            
            #generate the new temporary ranks and values
            ttrank = []
            ttvalue = []
            ttCPTcount = []
            for l in range (len(trank)):
                if trank[l] != actualrank:
                    ttrank.append(trank[l])
                    ttvalue.append(tvalue[l])
                    ttCPTcount.append(tCPTcount[l])
            for m in toadd:
                ttrank.append(actualrank -1)
                ttvalue.append(m)
                ttCPTcount.append(0)
            trank = ttrank
            tvalue = ttvalue
            tCPTcount = ttCPTcount
            actualrank -=1
    return result
    
def SearchChildAll(Child,ClusterDict,Species,ReducedTree):
    """
    Recursively find set of useful children
    """
    ChildKept = []
    if Child.data in ReducedTree:
        return [Child.data]
    for child in Child.children:
        ChildKept += SearchChild(child,ClusterDict,Species,ReducedTree)
    if Child.data in Species:
        ChildKept.insert(0,Child.data)
    if ChildKept and Child.data not in Species:
        ChildKept.append(Child.data)
    return ChildKept
    
def SearchChild(Child,ClusterDict,Species,ReducedTree):
    """
    Recursively find set of useful children
    """
    ChildKept = []
    if Child.data in ReducedTree:
        return [Child.data]
    if Child.data in Species:
        ChildKept.append(Child.data)
    for child in Child.children:
        ChildKept += SearchChild(child,ClusterDict,Species,ReducedTree)
    return ChildKept
"""            
def sum_proba_on_x(d,x,states):
    
    if not isinstance(d, dict): 
        return (d)
    elif (x==0):
        return [sum_proba_on_x(v,x-1,states) if d.has_key(v) else 0 for v in range(states)]
    elif (x>0):
        return np.sum([sum_proba_on_x(v,x-1,states) for v in d.itervalues()],0)
        # is making 5x more calculation than necessary
    else :
        return sum(sum_proba_on_x(v,x-1,states) for v in d.itervalues())
"""
def create_mult_dim_array(size,dimension):
    if dimension == 1:
        return [0.0 for n in range(size)]
    else :
        return [create_mult_dim_array(size,dimension-1)for n in range(size)]
        
def sum_proba_on_x(d,x,states):          
    dlist = literal_eval(d[0])
    dpos = []
    for i in x:
        dpos.append(dlist.index(i))
    newd = np.array(create_mult_dim_array(states,len(dpos)))
    for i in d[1]:
        nlist = powertolist(i[0],states,len(dlist))
        add = []
        for j in dpos:
            add.append(nlist[j])
        add = tuple(add)
        newd[add]+=i[1]
    return newd

def sum_proba_on_x_old(d,x,states):          
    dlist = literal_eval(d[0])
    dpos = dlist.index(x)
    newd = [0 for n in range(states)]
    for i in d[1]:
        nlist = powertolist(i[0],states,len(dlist))
        newd[nlist[dpos]] +=i[1]
    return newd
    
def Merge_dict(Dict1,Dict2):
    if not isinstance(Dict1, dict) and not isinstance(Dict2, dict):
        return Dict1+Dict2
    else :
        Dict = {}
        for Key in Dict1.keys():
            if Key in Dict2.keys():
                Dict[Key] = Merge_dict(Dict1[Key],Dict2[Key])
            else:
                Dict[Key] = Dict1[Key]
        for Key in Dict2.keys():
            if Key not in Dict1.keys():
                Dict[Key] = Dict2[Key]
        return Dict

def find_value_parent(ProbaTable,Counter,NewLink,states):
    ProbaLink = literal_eval(ProbaTable[0])
    Result = []
    for i in ProbaLink:
        Result.append(Counter[NewLink.index(i)])
    Position =listtopower(Result,states)
    for i in ProbaTable[1]:
        if i[0] == Position:
            return i[1]
    return 0
        
def RecursiveChangeCPT(ProbaTable,Parent,Position_parent,Link,NewLink,CPTs,Counter,states):
    """
    Recursively remove a part of a CPT using proba tables
    """
    NCPTs = {}
    Count = len(Counter)
    if Count == 0:
        for CPT in CPTs:
            NCPTs[CPT] = RecursiveChangeCPT(ProbaTable,Parent,Position_parent,Link,NewLink,CPTs[CPT],Counter+[CPT],states)
        return NCPTs
    if Count == len(NewLink): #end
        Lookproba = literal_eval(ProbaTable[0])
        Table = ProbaTable[1]
        Position = []
        
        for proba in Lookproba:
            Position.append(Counter[NewLink.index(proba)])#find where to search in table
        Position =listtopower(Position,states)
        Value = sum([x[1] if x[0]==Position else 0 for x in Table])
        PosParent = Position_parent
        for p in Parent :
            PosParent = PosParent[Counter[NewLink.index(p)]]
        #PosParent = Position_parent[Counter[NewLink.index(Parent)]]
        #PosParent = find_value_parent(ProbaTable,Counter,NewLink,states)
        if PosParent == 0 :
            return 0
        return Value*CPTs/PosParent
        
    elif NewLink[Count] in  literal_eval(ProbaTable[0]) and NewLink[Count] not in Parent : #remove
        
        NCPTs = {}
        for CPT in CPTs:
            N2 = RecursiveChangeCPT(ProbaTable,Parent,Position_parent,Link,NewLink,CPTs[CPT],Counter+[CPT],states)
            if NCPTs:
                NCPTs = Merge_dict(NCPTs,N2)
            else:
                NCPTs = N2
        return NCPTs
    
    elif NewLink[Count] in  literal_eval(ProbaTable[0]) and NewLink[Count] not in Link : #add
        Keep = False
        for i in range(states):
            NCPTs[i] = RecursiveChangeCPT(ProbaTable,Parent,Position_parent,Link,NewLink,CPTs,Counter+[i],states)
            if NCPTs[i]:
                Keep = True
        if Keep:
            return NCPTs
        else:
            return {}
    else : #normal
        for CPT in CPTs:
            NCPTs[CPT] = RecursiveChangeCPT(ProbaTable,Parent,Position_parent,Link,NewLink,CPTs[CPT],Counter+[CPT],states)
        return NCPTs

def RecursiveChangeCPTold(ProbaTable,Parent,Position_parent,Link,NewLink,CPTs,Counter,states):
    """
    Recursively remove a part of a CPT using proba tables
    """
    NCPTs = {}
    Count = len(Counter)
    if Count == 0:
        for CPT in CPTs:
            NCPTs[CPT] = RecursiveChangeCPT(ProbaTable,Parent,Position_parent,Link,NewLink,CPTs[CPT],Counter+[CPT],states)
        return NCPTs
    if Count == len(NewLink): #end
        Lookproba = literal_eval(ProbaTable[0])
        Table = ProbaTable[1]
        Position = []
        
        for proba in Lookproba:
            Position.append(Counter[NewLink.index(proba)])#find where to search in table
        Position =listtopower(Position,states)
        Value = sum([x[1] if x[0]==Position else 0 for x in Table])
        PosParent = Position_parent[Counter[NewLink.index(Parent)]]
        #PosParent = find_value_parent(ProbaTable,Counter,NewLink,states)
        if PosParent == 0 :
            return 0
        return Value*CPTs/PosParent
        
    elif NewLink[Count] in  literal_eval(ProbaTable[0]) and NewLink[Count] != Parent : #remove
        
        NCPTs = {}
        for CPT in CPTs:
            N2 = RecursiveChangeCPT(ProbaTable,Parent,Position_parent,Link,NewLink,CPTs[CPT],Counter+[CPT],states)
            if NCPTs:
                NCPTs = Merge_dict(NCPTs,N2)
            else:
                NCPTs = N2
        return NCPTs
    
    elif NewLink[Count] in  literal_eval(ProbaTable[0]) and NewLink[Count] not in Link : #add
        for i in range(states):
            NCPTs[i] = RecursiveChangeCPT(ProbaTable,Parent,Position_parent,Link,NewLink,CPTs,Counter+[i],states)
        return NCPTs    
    else : #normal
        for CPT in CPTs:
            NCPTs[CPT] = RecursiveChangeCPT(ProbaTable,Parent,Position_parent,Link,NewLink,CPTs[CPT],Counter+[CPT],states)
        return NCPTs
    
def ChangeCPTsOLD(ProbaTable,Parent,Link,CPTs,states):
    """
    Remove a part of a CPT using proba tables
    """
    NewLink = list(set(Link[1:]+literal_eval(ProbaTable[0])))
    NewLink.sort()
    Position_parent = sum_proba_on_x(ProbaTable,Parent,states)
    NewCPT = RecursiveChangeCPT(ProbaTable,Parent,Position_parent,["par"]+Link[1:],["par"]+NewLink,CPTs,[],states)
    for Value in literal_eval(ProbaTable[0]):
        if Value != Parent:
            NewLink.remove(Value)

    return [[Link[0]]+NewLink,NewCPT]
    
def ChangeCPTs(ProbaTable,Parent,Link,CPTs,states):
    """
    Remove a part of a CPT using proba tables
    """
    NewLink = list(set(Link[1:]+literal_eval(ProbaTable[0])))
    NewLink.sort()
    Position_parent = sum_proba_on_x(ProbaTable,Parent,states)
    NewCPT = RecursiveChangeCPT(ProbaTable,Parent,Position_parent,["par"]+Link[1:],["par"]+NewLink,CPTs,[],states)
    for Value in literal_eval(ProbaTable[0]):
        if Value not in Parent:
            NewLink.remove(Value)

    return [[Link[0]]+NewLink,NewCPT]    
    
def ReduceCPTs(ReducedTree,CPTs,Links,Cluster,ClusterDict,Species,Values,states):
    """
    Using the reduced tree, reduce the CPTs to fit the reduced tree
    """
    rank = []
    tree = []
    value = []
    for link in Links :
        if link[0] == Cluster[0]:
            link1 = link[1:]
        if len(Cluster) == 2:
            if link[0] == Cluster[1]:
                link2 = link[1:]
        else:
            link2 = []
    for specie in Species:
        rank.append(ClusterDict[specie].rank)
        tree.append(ClusterDict[specie].treenumber)
        value.append(ClusterDict[specie].data)
    for Node in ReducedTree:
        Children = ClusterDict[Node].children
        for Child in Children:
            if Child.data not in ReducedTree:
                ValidChild = SearchChild(Child,ClusterDict,Species,ReducedTree)
                if all(x in ReducedTree for x in ValidChild):
                    ValidChild = []
                if ValidChild:
                    VC = list(ValidChild)
                    ValidChild.append(Node)
                    ValidChild.sort()
                    # Search the good link
                    #if VC[0] in link1:
                    if all([V in link1 for V in VC]):
                        #NLinks = [[Cluster[0]]+ValidChild]
                        nlink = link1
                        NCluster = [Cluster[0]]
                    #elif VC[0] in link2:
                    elif all([V in link2 for V in VC]):
                        #NLinks = [[Cluster[1]]+ValidChild]
                        nlink = link2
                        NCluster = [Cluster[1]]
                    else :
                        sys.exit("FATAL ERROR : CPT Reduction failed")
                    ProbaTable = FindTreePath(CPTs,Links,NCluster,ClusterDict,ValidChild,Values,states)[0]
                    Parent =[]
                    for i in ValidChild:
                        if i in ReducedTree:
                            Parent.append(i)
                    #ProbaTable = FindTreePath(CPTs,NLinks,NCluster,ClusterDict,ValidChild,Values,states)[0]
                    [NLink, CPTs[NCluster[0]]] = ChangeCPTs(ProbaTable,Parent,NCluster+nlink,CPTs[NCluster[0]],states)
                    if all([V in link1 for V in VC]):
                        link1 = NLink[1:]
                    elif all([V in link2 for V in VC]):
                        link2 = NLink[1:]
                    else:
                        sys.exit("FATAL ERROR : CPT Reduction failed")
                    Links = [NLink if x[0]==NLink[0] else x for x in Links]
                    #TODO : Need to take into account new CPT reading.
    return [Links,CPTs]
    
def FindKeepValueOld(k,link1,link2,Cluster,CPT):
    KeepValues = []
    if k in link1:
        position = link1.index(k)
        for PossibleValue in CPT[Cluster[0]]:
            AValues = Seekvalues(CPT[Cluster[0]][PossibleValue],[position],0) #Search values authorized by CPT
            KeepValues.extend(AValues)
    else:
        if k in link2:
            position = link2.index(k)
            for PossibleValue in CPT[Cluster[1]]:
                AValues = Seekvalues(CPT[Cluster[1]][PossibleValue],[position],0) #Search values authorized by CPT
                KeepValues.extend(AValues)
    return [list(x) for x in set(tuple(x) for x in KeepValues)]
    
def FindKeepValue(k,link1,link2,Cluster,CPT):
    KeepValues = []
    if k in link1:
        position = link1.index(k)
        for PossibleValue in CPT[Cluster[0]]:
            AValues = Seekvalues(CPT[Cluster[0]][PossibleValue],[position],0) #Search values authorized by CPT
            KeepValues.extend(AValues)
    elif k in link2:
        position = link2.index(k)
        for PossibleValue in CPT[Cluster[1]]:
            AValues = Seekvalues(CPT[Cluster[1]][PossibleValue],[position],0) #Search values authorized by CPT
            KeepValues.extend(AValues)
    else:
        for i in range(6):
            KeepValues.append([i]) #TODO
    return [list(x) for x in set(tuple(x) for x in KeepValues)]
    
def FindKeepCluster(k,l,link1,link2,Cluster,CPT):
    KeepValues = []
    if k in link1 and l in link1:
        position = link1.index(k)
        position2 = link1.index(l)
        for PossibleValue in CPT[Cluster[0]]:
            AValues = Seekvalues(CPT[Cluster[0]][PossibleValue],[position,position2],0) #Search values authorized by CPT
            KeepValues.extend(AValues)
    else:
        if k in link2 and l in link2:
            position = link2.index(k)
            position2 = link2.index(l)
            for PossibleValue in CPT[Cluster[1]]:
                AValues = Seekvalues(CPT[Cluster[1]][PossibleValue],[position,position2],0) #Search values authorized by CPT
                KeepValues.extend(AValues)
        else:
            KeepValues = []
            Keepval1 = FindKeepValue(k,link1,link2,Cluster,CPT)
            Keepval2 = FindKeepValue(l,link1,link2,Cluster,CPT)
            for i in Keepval1:
                for j in Keepval2:
                    KeepValues.append(i+j)
            #KeepValues = FindKeepValue(k,link1,link2,Cluster,CPT)+FindKeepValue(l,link1,link2,Cluster,CPT)
    return [list(x) for x in set(tuple(x) for x in KeepValues)]
    
def Seekvalues(CPT,position,iteration):
    """
    Return values of species that exist in the CPT
    """
    if iteration == max(position):
        #if isinstance(CPT, dict):
        return [[x] for x in CPT.keys()]
        #else:
        #    return [[CPT]]
    else :
        RValues = []
        if iteration in position:
            Groupkeep = []
            for Value in CPT:
                Keep = []
                Results = Seekvalues(CPT[Value],position,iteration+1)
                for Result in Results:
                    Keep.append([Value]+Result)
                Groupkeep.extend(Keep)
            return [list(x) for x in set(tuple(x) for x in Groupkeep)]
        else:    
            for Value in CPT:
                RValues.extend(Seekvalues(CPT[Value],position,iteration+1))
        return [list(x) for x in set(tuple(x) for x in RValues)]
    

def FusionCluster(cluster1,cluster2,fusionpoint,Nbstates):
    """
    Fuse two sets of clusters containing the same specie. 
    """
    fusionvalue = fusionpoint[0]
    fusionprobas = fusionpoint[1]
    Cluster1list = literal_eval(cluster1[0])
    Cluster2list = literal_eval(cluster2[0])
    Nbvalues1 = len(Cluster1list)
    Nbvalues2 = len(Cluster2list)
    #Nbstates = len(cluster1[1])
    Newlist = list(set(Cluster1list+Cluster2list))
    Newlist.sort()
    Keypos1 = Cluster1list.index(fusionvalue)
    Keypos2 = Cluster2list.index(fusionvalue)
    #generate power multipliers for clusters values
    Power1 =[]
    for i in range(len(Cluster1list)):
        Power = Newlist.index(Cluster1list[i])
        Power1.append(pow(Nbstates,Power))
    Power2 =[]
    for i in range(len(Cluster2list)):
        Power = Newlist.index(Cluster2list[i])
        if i == Keypos2: 
            Power2.append(0)
        else:
            Power2.append(pow(Nbstates,Power))   
    dict1 ={} #lists results with same fusion value
    for i in range(len(cluster1[1])): #sort using fusion value.
        plist = powertolist(cluster1[1][i][0],Nbstates,Nbvalues1)
        NewCluster1val = 0 #generate new values using the ranks in the new cluster
        for j in range(len(plist)):
            NewCluster1val += plist[j]*Power1[j]
        proba = cluster1[1][i][1]
        keyvalue = plist[Keypos1]
        if dict1.has_key(keyvalue):
            dict1[keyvalue].append([NewCluster1val,proba])
        else:
            dict1[keyvalue] = [[NewCluster1val,proba]]         
    for i in dict1.keys(): # divide by common specie proba
        for j in range(len(dict1[i])):
            for k in fusionprobas:
                if k[0] == i:
                    divproba = k[1]
            if dict1[i][j][1] != 0:
                dict1[i][j][1]/= divproba
    dict2 = {} #lists results with same new table values
    for i in range(len(cluster2[1])): # fuse with cluster 1.
        plist = powertolist(cluster2[1][i][0],Nbstates,Nbvalues2)
        NewCluster2val = 0 #generate new values using the ranks in the new cluster
        for j in range(len(plist)):
            NewCluster2val += plist[j]*Power2[j]
        proba = cluster2[1][i][1]
        keyvalue = plist[Keypos2]
        if dict1.has_key(keyvalue):
            for j in range(len(dict1[keyvalue])):
                Newkey = dict1[keyvalue][j][0]+NewCluster2val
                Newproba = dict1[keyvalue][j][1]*proba
                if dict2.has_key(Newkey):
                    dict2[Newkey]=dict2[Newkey]*Newproba
                else:
                    dict2[Newkey]=Newproba
                #Newtable.append([Newvalues,Newproba])
                
    Newtable = []
    Tuple = dict2.items()
    for i in range(len(Tuple)):  
        Newtable.append([Tuple[i][0],Tuple[i][1]])
    clustertable =[str(Newlist),Newtable]
    return clustertable
    
    
def powertolist(number,base,values):
    """
    Transform a number to a list of it's component using a different base 
    """
    result = []
    for i in range(values):
        result.append(number%base)
        number /= base
    return result
    
def listtopower(nlist,base):
    """
    Transform a list of numbers to a base x generated number 
    """
    result = 0
    multiply = 1
    for i in range(len(nlist)):
        result += nlist[i]*multiply
        multiply *= base
    return result

def Marginalize(cluster,marginvalue,Nbstates):
    """
    Marginalize a table of species along one specie.
    """
    Clusterlist = literal_eval(cluster[0])
    Nbvalues = len(Clusterlist)
    Newlist = literal_eval(cluster[0])
    Keypos = Clusterlist.index(marginvalue)
    Newlist.remove(marginvalue)
    Power =[]
    Powerv = 0
    for i in range(len(Clusterlist)):
        if i == Keypos: 
            Power.append(0)
        else:
            Power.append(pow(Nbstates,Powerv)) 
            Powerv +=1
    dict = {}
    for i in range(len(cluster[1])): #sort using fusion value.
        plist = powertolist(cluster[1][i][0],Nbstates,Nbvalues)
        proba = cluster[1][i][1]
        NewClusterval = 0 #generate new values using the ranks in the new cluster
        for j in range(len(plist)):
            NewClusterval += plist[j]*Power[j]
        if dict.has_key(NewClusterval):
            dict[NewClusterval] = dict[NewClusterval] + proba
        else:
            dict[NewClusterval] = proba
    
    Newtable =[]
    Tuple = dict.items()
    for i in range(len(Tuple)):  
        Newtable.append([Tuple[i][0],Tuple[i][1]])
    clustertable =[str(Newlist),Newtable]
    return clustertable    

def CalculateProbaTables(RDict,ClusterDict,Cluster,Species,Values,CPTs,states,links):
    """   
    Find the relatives probabilities of different species using the cluster 
    tree
    """
    #TreeCPT = FindTreePath(CPTs,links,Cluster,ClusterDict,Species,Values,states)
    if len(Cluster) == 2:
        ReducedTree = FindReducedTree(links,Cluster,ClusterDict,Species)
        [links,CPTs] = ReduceCPTs(ReducedTree,CPTs,links,Cluster,ClusterDict,Species,Values,states)
        Species = ReducedTree
        Species.sort()        
    TreeCPT = FindTreePath(CPTs,links,Cluster,ClusterDict,Species,Values,states)
    # Changer FindTreePath pour prendre en compte CPTs
    result = []
    Newtree = RecursiveCalculateProba(TreeCPT,CPTs,Cluster,links,states)[0]
    #Newtree = ClearCPT(Newtree,CPTsAr,Cluster,links,states)
    for link in links :
        if link[0] == Cluster[0]:
            link1 = link[1:]
    if len(Cluster) == 2:
        for link in links :
            if link[0] == Cluster[1]:
                link2 = link[1:]
        poslink1 = [Species.index(x) for x in link1]
        poslink2 = [Species.index(x) for x in link2]
    #for Treevalue in Newtree[1]:    
    if len(Cluster) == 1:
        for i in CPTs[Cluster[0]]:
            valueresult = 0
            for k in Newtree[1]:
                ListValues = powertolist(int(k[0]),states,len(link1))
                KeepCPT=CPTs[Cluster[0]][i]
                existCPT = True
                position = 0
                while position < len(ListValues) and existCPT:
                    Value = ListValues[position]
                    if KeepCPT.has_key(Value):
                        KeepCPT=KeepCPT[Value]
                    else:
                        existCPT = False 
                    position += 1
                if existCPT:
                    proba = KeepCPT*k[1]
                    valueresult+=proba
            if valueresult > 0:
                result.append([i,valueresult])
    if len(Cluster) == 2:
        for i in CPTs[Cluster[0]]:
            for j in CPTs[Cluster[1]]:
                valueresult = 0
                for k in Newtree[1]:
                    ListValues = powertolist(int(k[0]),states,len(Species))
                    Values1 = [ListValues[x] for x in poslink1]
                    Values2 = [ListValues[x] for x in poslink2]
                    KeepCPT1=CPTs[Cluster[0]][i]
                    KeepCPT2=CPTs[Cluster[1]][j]
                    existCPT = True
                    position = 0
                    while position < len(ListValues) and existCPT:
                        Value = ListValues[position]
                        #repair needed : fit position 
                        if position in poslink1:
                            if KeepCPT1.has_key(Values1[poslink1.index(position)]):
                                KeepCPT1=KeepCPT1[Value]
                            else:
                                existCPT = False 
                        if position in poslink2:
                            if KeepCPT2.has_key(Values2[poslink2.index(position)]):
                                KeepCPT2=KeepCPT2[Value]
                            else:
                                existCPT = False 
                        position +=1    
                    if existCPT:
                        proba = KeepCPT1*KeepCPT2*k[1]
                        valueresult+=proba
                if valueresult > 0:
                    result.append([int(i)+int(j)*states,valueresult]) #TODO
    RDict[str(Cluster)]= result
    
def CalculateProbaTablesOld(ClusterDict,Cluster,Species,Values,CPTs,states,links):
    """   
    Find the relatives probabilities of different species using the cluster 
    tree
    """
    #TreeCPT = FindTreePath(CPTs,links,Cluster,ClusterDict,Species,Values,states)
    if len(Cluster) == 2:
        ReducedTree = FindReducedTree(links,Cluster,ClusterDict,Species)
        [links,CPTs] = ReduceCPTs(ReducedTree,CPTs,links,Cluster,ClusterDict,Species,Values,states)
        Species = ReducedTree
        Species.sort()
    TreeCPT = FindTreePath(CPTs,links,Cluster,ClusterDict,Species,Values,states)
    # Changer FindTreePath pour prendre en compte CPTs
    result = []
    Newtree = RecursiveCalculateProba(TreeCPT,CPTs,Cluster,links,states)[0]
    #Newtree = ClearCPT(Newtree,CPTsAr,Cluster,links,states)
    for link in links :
        if link[0] == Cluster[0]:
            link1 = link[1:]
    if len(Cluster) == 2:
        for link in links :
            if link[0] == Cluster[1]:
                link2 = link[1:]
        poslink1 = [Species.index(x) for x in link1]
        poslink2 = [Species.index(x) for x in link2]
    #for Treevalue in Newtree[1]:    
    if len(Cluster) == 1:
        for i in CPTs[Cluster[0]]:
            valueresult = 0
            for k in Newtree[1]:
                ListValues = powertolist(int(k[0]),states,len(link1))
                KeepCPT=CPTs[Cluster[0]][i]
                existCPT = True
                position = 0
                while position < len(ListValues) and existCPT:
                    Value = ListValues[position]
                    if KeepCPT.has_key(Value):
                        KeepCPT=KeepCPT[Value]
                    else:
                        existCPT = False 
                    position += 1
                if existCPT:
                    proba = KeepCPT*k[1]
                    valueresult+=proba
            result.append([i,valueresult])
    if len(Cluster) == 2:
        for i in CPTs[Cluster[0]]:
            for j in CPTs[Cluster[1]]:
                valueresult = 0
                for k in Newtree[1]:
                    ListValues = powertolist(int(k[0]),states,len(Species))
                    Values1 = [ListValues[x] for x in poslink1]
                    Values2 = [ListValues[x] for x in poslink2]
                    KeepCPT1=CPTs[Cluster[0]][i]
                    KeepCPT2=CPTs[Cluster[1]][j]
                    existCPT = True
                    position = 0
                    while position < len(ListValues) and existCPT:
                        Value = ListValues[position]
                        #repair needed : fit position 
                        if position in poslink1:
                            if KeepCPT1.has_key(Values1[poslink1.index(position)]):
                                KeepCPT1=KeepCPT1[Value]
                            else:
                                existCPT = False 
                        if position in poslink2:
                            if KeepCPT2.has_key(Values2[poslink2.index(position)]):
                                KeepCPT2=KeepCPT2[Value]
                            else:
                                existCPT = False 
                        position +=1
                        
                    if existCPT:
                        proba = KeepCPT1*KeepCPT2*k[1]
                        valueresult+=proba
                result.append([int(i)+int(j)*states,valueresult]) #TODO
    return [str(Cluster),result]

def ClearCPT(Treepath,CPTs,Cluster,Links,base):
    """
    Delete parts of the proba table that don't match CPTs
    """
    Values = literal_eval(Treepath[0])
    CPT = CPTs[Cluster[0]]
    for link in Links:
        if link[0]==Cluster[0]:
            Link = link[1:]
    if len(Cluster)==2:
        CPT2 = CPTs[Cluster[1]]
        for link in Links:
            if link[0]==Cluster[1]:
                Link2 = link[1:]
    Newset = []
    PosLink1 = []
    for x in Values:
        if x in Link:
            PosLink1.append(Link.index(x))
        else:
            PosLink1.append(-1)
    if len(Cluster)==2:
        PosLink1b = []
        for x in Values:
            if x in Link2:
                PosLink1b.append(Link2.index(x))
            else:
                PosLink1b.append(-1)
    for Set in Treepath[1]:
        Val1 = powertolist(Set[0],base,len(Values))
        if len(Cluster)==1:
            KeepCPT = CPT
            for i in range(len(Link)-1,-1,-1):
                if i in PosLink1 :
                    KeepCPT = KeepCPT.take(Val1[PosLink1.index(i)],i)
                else:
                    KeepCPT = KeepCPT.sum(i)
            Accept = KeepCPT
        else:
            KeepCPT = CPT
            KeepCPT2 = CPT2
            for i in range(len(Link)-1,-1,-1):
                if i in PosLink1 :
                    KeepCPT = KeepCPT.take(Val1[PosLink1.index(i)],i)   
                else:
                    KeepCPT = KeepCPT.sum(i)
            for i in range(len(Link2)-1,-1,-1):
                if i in PosLink1b :
                    KeepCPT2 = KeepCPT2.take(Val1[PosLink1b.index(i)],i)   
                else:
                    KeepCPT2 = KeepCPT2.sum(i)
            Accept = KeepCPT * KeepCPT2
        if Accept:            
            Newset.append(Set)
    return [Treepath[0],Newset]

def Recursive_keep_CPT(CPT,Val1,PosLink1,Val2,PosLink2,rank):
    if rank == 0:
        for Key in CPT.keys():
            if Recursive_keep_CPT(CPT[Key],Val1,PosLink1,Val2,PosLink2,rank+1):
                return True
    if not isinstance(CPT, dict):
        if CPT > 0:
            return True
        return False
    if rank-1 in PosLink1:
        Val = Val1[PosLink1.index(rank-1)]
        if CPT.has_key(Val):
            return Recursive_keep_CPT(CPT[Val],Val1,PosLink1,Val2,PosLink2,rank+1)
        return False
    if rank-1 in PosLink2:
        Val = Val2[PosLink2.index(rank-1)]
        if CPT.has_key(Val):
            return Recursive_keep_CPT(CPT[Val],Val1,PosLink1,Val2,PosLink2,rank+1)
        return False
    for Key in CPT.keys():
        if Recursive_keep_CPT(CPT[Key],Val1,PosLink1,Val2,PosLink2,rank+1):
            return True
    return False
    
def RecursiveCalculateProba(Treepath,CPTs,Cluster,Links,base):
    """   
    Merge probabilities for each tree into one big set of probabilities. 
    """
    if len(Treepath)==1:
        return Treepath
    else:
        Values = literal_eval(Treepath[0][0])
        CPT = CPTs[Cluster[0]]
        for link in Links:
            if link[0]==Cluster[0]:
                Link = link[1:]
        if len(Cluster)==2:
            CPT2 = CPTs[Cluster[1]]
            for link in Links:
                if link[0]==Cluster[1]:
                    Link2 = link[1:]
        Tree2 = RecursiveCalculateProba(Treepath[1:],CPTs,Cluster,Links,base)
        Values2 = literal_eval(Tree2[0][0])
        Newset = []
        Newvalues = Values+Values2
        Newvalues.sort()
        Pos1 = [Newvalues.index(x) for x in Values]
        Pos2 = [Newvalues.index(x) for x in Values2]
        PosLink1 = []
        for x in Values:
            if x in Link:
                PosLink1.append(Link.index(x))
            else:
                PosLink1.append(-1)
        #PosLink1 = [Link.index(x) for x in Values]
        PosLink2 = []
        for x in Values2:
            if x in Link:
                PosLink2.append(Link.index(x))
            else:
                PosLink2.append(-1)
        #PosLink2 = [Link.index(x) for x in Values2]
        if len(Cluster)==2:
            #PosLink1b = [Link2.index(x) for x in Values]
            PosLink1b = []
            for x in Values:
                if x in Link2:
                    PosLink1b.append(Link2.index(x))
                else:
                    PosLink1b.append(-1)
            #PosLink2b = [Link2.index(x) for x in Values2]
            PosLink2b = []
            for x in Values2:
                if x in Link2:
                    PosLink2b.append(Link2.index(x))
                else:
                    PosLink2b.append(-1)
        for Set in Treepath[0][1]:
            Val1 = powertolist(Set[0],base,len(Values))
            for Set2 in Tree2[0][1]:
                Val2 = powertolist(Set2[0],base,len(Values2))
                if len(Cluster)==1:
                    Accept = Recursive_keep_CPT(CPT,Val1,PosLink1,Val2,PosLink2,0)
                else:   
                    Accept1 = Recursive_keep_CPT(CPT,Val1,PosLink1,Val2,PosLink2,0)
                    Accept2 = Recursive_keep_CPT(CPT2,Val1,PosLink1b,Val2,PosLink2b,0)
                    Accept = Accept1 * Accept2
                if Accept:            
                    Newval = [0 for x in Newvalues]
                    for i in range(len(Pos1)):
                        Newval[Pos1[i]]=Val1[i]
                    for j in range(len(Pos2)):
                        Newval[Pos2[j]]=Val2[j]
                    Newval = listtopower(Newval,base)
                    Newset.append([Newval,Set[1]*Set2[1]])
        return [[str(Newvalues),Newset]]
        
def RecursiveCalculateProbaOld(Treepath,CPTs,Cluster,Links,base):
    """   
    Merge probabilities for each tree into one big set of probabilities. 
    """
    if len(Treepath)==1:
        return Treepath
    else:
        Values = literal_eval(Treepath[0][0])
        CPT = CPTs[Cluster[0]]
        for link in Links:
            if link[0]==Cluster[0]:
                Link = link[1:]
        if len(Cluster)==2:
            CPT2 = CPTs[Cluster[1]]
            for link in Links:
                if link[0]==Cluster[1]:
                    Link2 = link[1:]
        Tree2 = RecursiveCalculateProba(Treepath[1:],CPTs,Cluster,Links,base)
        Values2 = literal_eval(Tree2[0][0])
        Newset = []
        Newvalues = Values+Values2
        Newvalues.sort()
        Pos1 = [Newvalues.index(x) for x in Values]
        Pos2 = [Newvalues.index(x) for x in Values2]
        PosLink1 = []
        for x in Values:
            if x in Link:
                PosLink1.append(Link.index(x))
            else:
                PosLink1.append(-1)
        #PosLink1 = [Link.index(x) for x in Values]
        PosLink2 = []
        for x in Values2:
            if x in Link:
                PosLink2.append(Link.index(x))
            else:
                PosLink2.append(-1)
        #PosLink2 = [Link.index(x) for x in Values2]
        if len(Cluster)==2:
            #PosLink1b = [Link2.index(x) for x in Values]
            PosLink1b = []
            for x in Values:
                if x in Link2:
                    PosLink1b.append(Link2.index(x))
                else:
                    PosLink1b.append(-1)
            #PosLink2b = [Link2.index(x) for x in Values2]
            PosLink2b = []
            for x in Values2:
                if x in Link2:
                    PosLink2b.append(Link2.index(x))
                else:
                    PosLink2b.append(-1)
        for Set in Treepath[0][1]:
            Val1 = powertolist(Set[0],base,len(Values))
            for Set2 in Tree2[0][1]:
                Val2 = powertolist(Set2[0],base,len(Values2))
                if len(Cluster)==1:
                    KeepCPT = CPT
                    for i in range(len(Link)-1,-1,-1):
                        if i in PosLink1:
                            KeepCPT = KeepCPT.take(Val1[PosLink1.index(i)],i)
                        elif i in PosLink2 :
                            KeepCPT = KeepCPT.take(Val2[PosLink2.index(i)],i)
                        else:
                            KeepCPT = KeepCPT.sum(i)
                    Accept = KeepCPT
                else:
                    KeepCPT = CPT
                    KeepCPT2 = CPT2
                    for i in range(len(Link)-1,-1,-1):
                        if i in PosLink1:
                            KeepCPT = KeepCPT.take(Val1[PosLink1.index(i)],i)   
                        elif i in PosLink2 :
                            KeepCPT = KeepCPT.take(Val2[PosLink2.index(i)],i)
                        else:
                            KeepCPT = KeepCPT.sum(i)
                    for i in range(len(Link2)-1,-1,-1):
                        if i in PosLink1b :
                            KeepCPT2 = KeepCPT2.take(Val1[PosLink1b.index(i)],i)   
                        elif i in PosLink2b :
                            KeepCPT2 = KeepCPT2.take(Val2[PosLink2b.index(i)],i)
                        else:
                            KeepCPT2 = KeepCPT2.sum(i)
                    Accept = KeepCPT * KeepCPT2
                if Accept:            
                    Newval = [0 for x in Newvalues]
                    for i in range(len(Pos1)):
                        Newval[Pos1[i]]=Val1[i]
                    for j in range(len(Pos2)):
                        Newval[Pos2[j]]=Val2[j]
                    Newval = listtopower(Newval,base)
                    Newset.append([Newval,Set[1]*Set2[1]])
        return [[str(Newvalues),Newset]]
         
def multi_run_wrapper(i,args):
   return CalculateProbaTables(i,*args)   
   
def ParallelResults(Groupedclusters,cluster,Newlink,Values,CPTs,states,Links):
    return 0
   
def searchparents(Values,CPTdict,Links,Clusterdir,time,speciesnames,states):
    """   
    Generate the new table at t using the CPTs and the table at t-1.
    """  
    #Values = readtable(Resultdir+'T'+str(time-1)+'.txt')
    Clusters = readtable(Clusterdir+'T'+str(time-1)+'.txt')
    for cluster in Clusters:
        cluster.sort()
    ClustersT = readtable(Clusterdir+'T'+str(time)+'.txt')
    for cluster in ClustersT:
        cluster.sort()
    CPTs = CPTdict[str(time)]
    #CPTsAr = CPTarray[str(time)]
    #Groupedclusters = Groupclusters(Clusters)
    Groupedclusters = GenerateClusterTree(Clusters) #group clusters as trees
    FindSeparatedTrees(Groupedclusters) #give a rank number and a tree number
    # generate table containing all results
    Results = {}
    ClustersT2 = []
    for i in speciesnames: 
        ClustersT2.append([i])
    for cluster in ClustersT: 
        if len(cluster) > 1:
            ClustersT2.append(cluster)
    ParallelJob = []
    for cluster in ClustersT2: 
        Newlink = []
        NewCPT = {}
        for specie in cluster:
            NewCPT[specie]=copy.deepcopy(CPTs[specie])
            for link in Links:
                if link[0] == specie:
                    for value in link[1:]:
                        if not value in Newlink:
                            Newlink.append(value) # store all necessary values to calculate the cluster
        Newlink.sort()
        #print "Cluster : "+str(cluster)
        ParallelJob.append((copy.deepcopy(Groupedclusters),cluster,Newlink,copy.deepcopy(Values),NewCPT,states,Links))
        #Result = CalculateProbaTables(Groupedclusters,cluster,Newlink,Values,copy.deepcopy(CPTs),states,Links) # search the probability table.
        #Results[Result[0]] = Result[1] 
    #"""
    for i in ParallelJob: # not parallel
        multi_run_wrapper(Results, i)
    """# Parallel
    if __name__ == '__main__':
        manager = Manager()
        Results = manager.dict()
        job = [Process(target=multi_run_wrapper, args=(Results, i)) for i in ParallelJob]
        _ = [p.start() for p in job]
        _ = [p.join() for p in job]
        #p = Pool(5)
        #print (p.map(multi_run_wrapper,ParallelJob))
    """
    return CalculateIndividualProbas(Results,states) # Add individual probas and check if they match the clusters
    
def CalculateIndividualProbas(Results,base):
    """
    Calculate the individuals probabilities of species of a set of cluster by marginalizing them
    """
    
    for strcluster in Results.keys():
        cluster = literal_eval(strcluster)
        if type(cluster)!= type(str()) and len (cluster) == 1:
            Results[strcluster]= TestHundredPercent(Results[strcluster]) 
            Results[str(cluster[0])]= Results[strcluster]
    for strcluster in Results.keys():
        try :
            cluster = literal_eval(strcluster)
        except ValueError :
            cluster = strcluster            
        Results[strcluster]= TestHundredPercent(Results[strcluster]) 
        if type(cluster)== type([]) and len (cluster) == 2 :
            CValue1 = str([cluster[0]])
            CValue2 = str([cluster[1]])
            Key1 = Results[CValue1]
            Key2 = Results[CValue2]
            Value1 = {}
            Value2 = {}
            for Result in Results[strcluster]: # search marginal probabilities
                Value = powertolist(int(Result[0]),base,len(cluster))
                if Value1.has_key(Value[0]):
                    Value1[Value[0]] = Value1[Value[0]]+ Result[1]
                else:
                    Value1[Value[0]] = Result[1]
                if Value2.has_key(Value[1]):
                    Value2[Value[1]] = Value2[Value[1]]+ Result[1]
                else:
                    Value2[Value[1]] = Result[1] 
            repair = False
            for Key in Key1: #compare to probabilities for the species
                if Value1.has_key(Key[0]):
                    if round(Key[1],5) - round(Value1[Key[0]],5) > 0.0001:
                        repair = True
            for Key in Key2: 
                if Value2.has_key(Key[0]):
                    if round(Key[1],5) - round(Value2[Key[0]],5) > 0.0001:
                        repair = True
            if repair :
                #maxstuff = Key[1]-Value1[Key[0]]
                ResCluster = Results[strcluster]
                ResCN = [[0 for x in range(base)] for x in range(base)]
                for i in ResCluster:
                    pos = powertolist(i[0],base,len(cluster))
                    ResCN[pos[0]][pos[1]]=i[1]
                ResCN = np.array(ResCN)
                Alpha = Results[CValue1]
                AlphaN = [0 for x in range(base)]
                for i in Alpha:
                    AlphaN[i[0]] = i[1]
                AlphaN = np.array(AlphaN)
                Beta = Results[CValue2]
                BetaN = [0 for x in range(base)]
                for i in Beta:
                    BetaN[i[0]] = i[1]
                BetaN = np.array(BetaN)
                Error = 0.0001
                Newtable = FitMatrix.ipfp(ResCN,AlphaN,BetaN,Error)
                Newresults = []
                for i in range(len(Newtable)): # transform back
                    for j in range(len(Newtable[i])):
                        if Newtable[i][j]!=0:
                            value = listtopower([i,j],base)
                            proba = Newtable[i][j]
                            Newresults.append([value,proba]) 
                Results[strcluster] = Newresults
    return Results

    
def TestHundredPercent(vplist):
    """
    Take a list of [value, proba] and make the probabilities add up to 1.
    """
    totalproba = 0.0
    for item in vplist:
        totalproba = totalproba + item[1]
    if totalproba == 0 :
        return vplist
    else:
        multiply = 1.0/totalproba
    newlist = []
    for item in vplist: 
        newlist.append([item[0],item[1]*multiply])
    return newlist
    
    
def recursivewriteresults(writingtable,table,nbspecies,result,path):
    """
    Write the results in the given table.
    """
    count = 0
    if nbspecies < len(table)-1:
        for i in table[nbspecies]:
            recursivewriteresults(writingtable,table,nbspecies+1,i*result,path+str(count)+' ')
            count+=1
    else:
        for i in table[nbspecies]:
            writingtable.append([path+str(count)+' ',i*result])
            count+=1

def recursivereadCPT(CPTvalue,result,states,species):
    """
    Given a CPT value return an array containing the state of each specie
    """
    result.append(CPTvalue%states)
    CPTvalue /= states
    if species > 0:
        recursivereadCPT(CPTvalue,result,states,species-1)
    return result

def linkedvalues(link,Values,CPT):
    """   
    Search probabilities of having a set of values and return it.
    """  
    Percent = -1
    for Value in Values:
        if Value[0] in link[1:]:
            if int(Value[1]) == CPT[link[1:].index(Value[0])]:
                if Percent == -1:
                    Percent = 1.0
                Percent *= float(Value[2])
    if Percent == -1:
        return 0
    return Percent

def GenerateCPTdict(CPTsdir,CPTfile,States,Links):
    """   
    Generate a dictionnary containing all cpts.
    """
    CPTsread = open(CPTsdir+CPTfile, 'rU')
    CPTslines = CPTsread.readlines()
    CPTsread.close()
    CPTsdict = {}
    for Line in CPTslines:
        CPT = re.split(',',Line) [:-1]
        Dict = {}
        for link in Links:
            if link[0] == CPT[1]:
                Keeplink = link
        for Value in range(int(CPT[3])):
            Dict = RecursiveCPTdict(int(CPT[4+Value*2]),float(CPT[5+Value*2]),Dict,States,len(Keeplink)-2)
        if CPTsdict.has_key(CPT[0]):
            if CPTsdict[CPT[0]].has_key(CPT[1]):
                CPTsdict[CPT[0]][CPT[1]][int(CPT[2])]=Dict
            else:
                CPTsdict[CPT[0]][CPT[1]]={int(CPT[2]):Dict}
        else:
            CPTsdict[CPT[0]]={CPT[1]:{int(CPT[2]):Dict}}
    return CPTsdict
    
def GenerateCPTarray(CPTsdir,CPTfile,States,Links):
    """   
    Generate a dictionnary of arrays describing existing CPTs.
    """
    CPTsread = open(CPTsdir+CPTfile, 'rU')
    CPTslines = CPTsread.readlines()
    CPTsread.close()
    CPTsdict = {}
    for Line in CPTslines:
        CPT = re.split(',',Line) [:-1]
        for link in Links:
            if link[0] == CPT[1]:
                Keeplink = link
        if not CPTsdict.has_key(CPT[0]):
            CPTsdict[CPT[0]]={}
        if not CPTsdict[CPT[0]].has_key(CPT[1]):   
            Array = [0]*States
            for i in range(len(Keeplink)-2):
                Array = [[x]*States for x in Array]
            Array = np.array(Array)
            CPTsdict[CPT[0]][CPT[1]]=Array    
        for Value in range(int(CPT[3])):
            Position = powertolist(int(CPT[4+Value*2]),States,len(Keeplink)-1)
            Position = tuple(Position)
            CPTsdict[CPT[0]][CPT[1]][Position] = 1

    return CPTsdict
    
def RecursiveCPTdict(CPT,Proba,Dict,States,Iter):
    if Iter < 1:
        if not Dict.has_key(CPT%States):   
            Dict[CPT%States]=Proba
            return Dict
        else:
            print (CPT)
            print (Proba)
            print (Dict)
            sys.exit('ERROR: redundant CPT')
    else:
        if not Dict.has_key(CPT%States):
            Newdict = {}
            Dict[CPT%States]= RecursiveCPTdict(CPT/States,Proba,Newdict,States,Iter-1)           
        else:
            Dict[CPT%States]= RecursiveCPTdict(CPT/States,Proba,Dict[CPT%States],States,Iter-1)
        return Dict

def initialclusters(init,Clusterdir,states):
    """   
    Generate values for initial clusters
    """
    Clusterdict = {}
    initread= open(init,'r')
    initlines = initread.readlines()
    initread.close()
    initvalues = []
    #Clusters = readtable(Clusterdir+'T0.txt')
    for i in initlines:
        ilist = i.split()
        initvalues.append
        #states = len(ilist)-1
        proba =[]
        for i in range(len(ilist)-1) :
            proba.append([i,float(ilist[i+1])])
        Clusterdict[ilist[0]]= proba
    return Clusterdict

    
def savedatas(tempresults,Savedir,speciesnames,states,first,timepoint):
    """
    generate data used to trace curves of species evolution
    """
    if not os.path.isdir(Savedir):
        os.mkdir(Savedir)
    if not first and os.path.isfile(Savedir+'marginals.txt'):
        Resultfile = open(Savedir+'marginals.txt', 'r')
        Readfile = Resultfile.read()
        Resultfile.close()
    toadd = ''
    for i in speciesnames:
        toadd += str(timepoint)+' '+i
        for j in range(states):
            found = False
            for result in tempresults[i]:
                if result[0] == j :
                    toadd += ' '+ str(result[1])
                    found =True
            if not found:
                toadd += ' 0'
        toadd += '\n'
    Resultfile = open(Savedir+'marginals.txt', 'w+')
    if first :
        Readfile = toadd
    else:
        Readfile = Readfile+ toadd
    Resultfile.write(Readfile)
    Resultfile.close()
        
            
    '''
    for i in range(species):
        if not first and os.path.isfile(Savedir+str(i)+'.txt'):
            Resultfile = open(Savedir+str(i)+'.txt', 'r')
            Readfile = Resultfile.read()
            Resultfile.close()
        else:
            Readfile = ''
        Resultfile = open(Savedir+str(i)+'.txt', 'w+')
        Results = tempresults[str(i)]
        Value = 0
        Toadd =[]
        for result in Results:
            Value += (result[0]+1)*result[1]
        if first :
            Readfile = str(Value) 
        else:
            Readfile = Readfile+ ','  + str(Value) 
        Resultfile.write(Readfile)
        Resultfile.close()
    '''
def findspecies(Links):
    species = []
    for link in Links:
        species.append(link[0])
    return species
    
start_time = time.time()    
def main(): 
    """   
    Main function
    """
    args = sys.argv[1:]
    if len(args) == 3:
        path = args[0]
        timepoints = int(args[1])
        states = int(args[2])
    else :
        sys.exit("Require 3 arguments: a path, a number of time points and an number of states")

    #path = "/Users/mpichene/Desktop/Datas/Brown/" #change later
    #path = "/Users/mpichene/Desktop/Datas/Trail/"
    #path = "/Users/mpichene/Desktop/Enzkinetics_newcpt/data_files/" #change later
    #path = "/Users/mpichene/Desktop/Cluster/Deathexample/"
    init = path + "init_Marginal.csv"
    Savedir = path+"save/"
    CPTfile = 'cpt.csv'
    Links = readtable(path+'structure.txt')
    for link in Links :
        link[1:] = sorted(link[1:])
    Clusterdir = path + "clusters/"
    
    species = findspecies(Links)
    #states = 5 

    #initialvalues(Resultdir,init) # Generate the first results and write them
    CPTdict = GenerateCPTdict(path,CPTfile,states,Links) #Generate CPT file
    #CPTarray = GenerateCPTarray(CPTsdir,CPTfile,states,Links) #Generate a second file used to check if CPTs containing specific values exist
    tempresults = initialclusters(init,Clusterdir,states) #generate the first clusters probas$
    savedatas(tempresults,Savedir,species,states,True,0)
    #tempresults = readtable(Resultdir+'T0.txt')
    for i in range(1,timepoints+1): # loop to generate results. clusters probas are updated
        print i
        #print tempresults
        #tempresults = searchparentsold(tempresults,Resultdir,Tablesdir,Linkdir,Clusterdir,i,4,5)
        tempresults = searchparents(tempresults,CPTdict,Links,Clusterdir,i,species,states)
        savedatas(tempresults,Savedir,species,states,False,i)
        #print tempresults
        #print 'tempresults'
        #print tempresults
        print("--- %s seconds ---" % (time.time() - start_time))
if __name__ == '__main__':
    main()